@@ -78,8 +78,24 @@ private:
         ~block();
     };
 
+    struct element_block_deleter : public std::unary_function<void, const element_block_type*>
+    {
+        void operator() (const element_block_type* p)
+        {
+            element_block_func::delete_block(p);
+        }
+    };
+
     typedef std::vector<block*> blocks_type;
 
+    struct blocks_to_transfer
+    {
+        blocks_type blocks;
+        size_type insert_index;
+
+        blocks_to_transfer();
+    };
+
     struct iterator_trait
     {
         typedef multi_type_vector parent;
@@ -117,8 +133,13 @@ public:
     typedef __mtv::iterator_base<iterator_trait, itr_forward_update> iterator;
     typedef __mtv::iterator_base<reverse_iterator_trait, itr_no_update> reverse_iterator;
 
-    typedef __mtv::const_iterator_base<const_iterator_trait, iterator> const_iterator;
-    typedef __mtv::const_iterator_base<const_reverse_iterator_trait, reverse_iterator> const_reverse_iterator;
+    typedef __mtv::const_iterator_base<const_iterator_trait, itr_forward_update, iterator> const_iterator;
+    typedef __mtv::const_iterator_base<const_reverse_iterator_trait, itr_no_update, reverse_iterator> const_reverse_iterator;
+
+    typedef itr_node value_type;
+
+    typedef std::pair<iterator, size_type> position_type;
+    typedef std::pair<const_iterator, size_type> const_position_type;
 
     iterator begin();
     iterator end();
@@ -373,7 +394,7 @@ public:
     /**
      * Return the value of an element at specified position and set that
      * position empty.  If the element resides in a managed block, this call
-     * will <i>not</i> delete that element.  If the element is on a
+     * will release that element from that block.  If the element is on a
      * non-managed block, this call is equivalent to set_empty(pos, pos).
      *
      * <p>The method will throw an <code>std::out_of_range</code> exception if
@@ -386,6 +407,51 @@ public:
     template<typename _T>
     _T release(size_type pos);
 
+    /**
+     * Retrieve the value of an element at specified position and set that
+     * position empty.  If the element resides in a managed block, this call
+     * will release that element from that block.  If the element is on a
+     * non-managed block, this call is equivalent to set_empty(pos, pos).
+     *
+     * <p>The method will throw an <code>std::out_of_range</code> exception if
+     * the specified position is outside the current container range.</p>
+     *
+     * @param pos position of the element to release.
+     * @param value element value.
+     *
+     * @return iterator referencing the block where the position of the
+     *         released element is.
+     */
+    template<typename _T>
+    iterator release(size_type pos, _T& value);
+
+    /**
+     * Retrieve the value of an element at specified position and set that
+     * position empty.  If the element resides in a managed block, this call
+     * will release that element from that block.  If the element is on a
+     * non-managed block, this call is equivalent to set_empty(pos, pos).
+     *
+     * <p>The method will throw an <code>std::out_of_range</code> exception if
+     * the specified position is outside the current container range.</p>
+     *
+     * @param pos position of the element to release.
+     * @param value element value.
+     *
+     * @return iterator referencing the block where the position of the
+     *         released element is.
+     */
+    template<typename _T>
+    iterator release(const iterator& pos_hint, size_type pos, _T& value);
+
+    /**
+     * Release all its elements, and empties its content.  Calling this method
+     * relinquishes the ownership of all elements stored in managed blocks.
+     *
+     * <p>This call is equivalent of clear() if the container consists of no
+     * managed blocks.</p>
+     */
+    void release();
+
     /**
      * Given the logical position of an element, get the iterator of the block
      * where the element is located, and its offset from the first element of
@@ -398,7 +464,7 @@ public:
      * @return iterator referencing the block where the element resides, and
      *         its offset within the block.
      */
-    std::pair<iterator, size_type> position(size_type pos);
+    position_type position(size_type pos);
 
     /**
      * Given the logical position of an element, get the iterator of the block
@@ -415,7 +481,7 @@ public:
      * @return iterator referencing the block where the element resides, and
      *         its offset within the block.
      */
-    std::pair<iterator, size_type> position(const iterator& pos_hint, size_type pos);
+    position_type position(const iterator& pos_hint, size_type pos);
 
     /**
      * Given the logical position of an element, get the iterator of the block
@@ -429,7 +495,24 @@ public:
      * @return iterator referencing the block where the element resides, and
      *         its offset within the block.
      */
-    std::pair<const_iterator, size_type> position(size_type pos) const;
+    const_position_type position(size_type pos) const;
+
+    /**
+     * Given the logical position of an element, get the iterator of the block
+     * where the element is located, and its offset from the first element of
+     * that block.
+     *
+     * <p>The method will throw an <code>std::out_of_range</code> exception if
+     * the specified position is outside the current container range.</p>
+     *
+     * @param pos_hint iterator used as a block position hint, to specify
+     *                 which block to start when searching for the element
+     *                 position.
+     * @param pos logical position of the element.
+     * @return iterator referencing the block where the element resides, and
+     *         its offset within the block.
+     */
+    const_position_type position(const const_iterator& pos_hint, size_type pos) const;
 
     /**
      * Move elements from one container to another. After the move, the
@@ -671,6 +754,16 @@ public:
      */
     void swap(multi_type_vector& other);
 
+    /**
+     * Swap a part of the content with another container.
+     *
+     * @param start_pos starting position
+     * @param end_pos ending position, inclusive.
+     * @param other another container to swap the content with.
+     * @param other_pos insertion position in the other container.
+     */
+    void swap(size_type start_pos, size_type end_pos, multi_type_vector& other, size_type other_pos);
+
     bool operator== (const multi_type_vector& other) const;
     bool operator!= (const multi_type_vector& other) const;
 
@@ -697,6 +790,9 @@ private:
     template<typename _T>
     iterator set_impl(size_type pos, size_type start_row, size_type block_index, const _T& value);
 
+    template<typename _T>
+    iterator release_impl(size_type pos, size_type start_pos, size_type block_index, _T& value);
+
     /**
      * Find the correct block position for given logical row ID.
      *
@@ -722,7 +818,7 @@ private:
      * Same as above, but try to infer block position from the iterator first
      * before trying full search.
      */
-    void get_block_position(const iterator& pos_hint, size_type pos, size_type& start_pos, size_type& block_index) const;
+    void get_block_position(const const_iterator& pos_hint, size_type pos, size_type& start_pos, size_type& block_index) const;
 
     template<typename _T>
     static void create_new_block_with_new_cell(element_block_type*& data, const _T& cell);
@@ -754,12 +850,45 @@ private:
         size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1,
         multi_type_vector& dest, size_type dest_pos);
 
+    /**
+     * All elements to transfer to the other container is in the same block.
+     */
     iterator transfer_single_block(
         size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1,
         multi_type_vector& dest, size_type dest_pos);
 
+    /**
+     * Elements to transfer to the other container span across multiple
+     * blocks.
+     */
+    iterator transfer_multi_blocks(
+        size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1,
+        size_type start_pos_in_block2, size_type block_index2,
+        multi_type_vector& dest, size_type dest_pos);
+
     iterator set_empty_impl(size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1);
 
+    void swap_impl(
+        multi_type_vector& other, size_type start_pos, size_type end_pos, size_type other_pos,
+        size_type start_pos_in_block1, size_type block_index1, size_type start_pos_in_block2, size_type block_index2,
+        size_type start_pos_in_dblock1, size_type dblock_index1, size_type start_pos_in_dblock2, size_type dblock_index2);
+
+    void swap_single_blocks(
+        multi_type_vector& other, size_type start_pos, size_type end_pos, size_type other_pos,
+        size_type start_pos_in_block, size_type block_index, size_type start_pos_in_other_block, size_type other_block_index);
+
+    void swap_single_to_multi_blocks(
+        multi_type_vector& other, size_type start_pos, size_type end_pos, size_type other_pos,
+        size_type start_pos_in_block, size_type block_index, size_type dst_start_pos_in_block1, size_type dst_block_index1,
+        size_type dst_start_pos_in_block2, size_type dst_block_index2);
+
+    void swap_multi_to_multi_blocks(
+        multi_type_vector& other, size_type start_pos, size_type end_pos, size_type other_pos,
+        size_type start_pos_in_block1, size_type block_index1, size_type start_pos_in_block2, size_type block_index2,
+        size_type start_pos_in_dblock1, size_type dblock_index1, size_type start_pos_in_dblock2, size_type dblock_index2);
+
+    void prepare_blocks_to_transfer(blocks_to_transfer& bucket, size_type block_index1, size_type offset1, size_type block_index2, size_type offset2);
+
     iterator set_whole_block_empty(size_type block_index, size_type start_pos_in_block, bool overwrite);
 
     iterator set_empty_in_single_block(
@@ -843,6 +972,50 @@ private:
         size_type row, size_type block_index, size_type start_pos,
         const _T& it_begin, const _T& it_end);
 
+    /**
+     * Set a new block in the middle of an existing block. This call inserts
+     * two new blocks below the specificed block position. The first one will
+     * be empty, and the second one will contain the lower elements of the
+     * existing block.
+     *
+     * @param block_index index of block into which to set a new block.
+     * @param offset position in the existing block to set the new block to.
+     * @param new_block_size size of the new block
+     * @param overwrite whether or not to overwrite the elements replaced by
+     *                  the new block.
+     * @return pointer to the middle block
+     */
+    block* set_new_block_to_middle(
+        size_type block_index, size_type offset, size_type new_block_size, bool overwrite);
+
+    block* get_previous_block_of_type(size_type block_index, element_category_type cat);
+    block* get_next_block_of_type(size_type block_index, element_category_type cat);
+
+    /**
+     * Send elements from a source block to place them in a destination block.
+     * In return, the method returns the elements in the destination block
+     * that have been replaced by the elements sent by the caller.  The caller
+     * needs to manage the life cycle of the returned block.
+     *
+     * @param src_data source data block from which the elements are sent.
+     * @param src_offset position of the first element in the source block.
+     * @param dst_index destination block index.
+     * @param dst_offset position in the destination block where the sent
+     *                   elements are to be placed.
+     * @param len length of elements.
+     *
+     * @return heap allocated block that contains the overwritten elements
+     *         originally in the destination block. The caller needs to manage
+     *         its life cycle.
+     */
+    element_block_type* exchange_elements(
+        const element_block_type& src_data, size_type src_offset, size_type dst_index, size_type dst_offset, size_type len);
+
+    void exchange_elements(
+        const element_block_type& src_data, size_type src_offset,
+        size_type dst_index1, size_type dst_offset1, size_type dst_index2, size_type dst_offset2,
+        size_type len, blocks_type& new_blocks);
+
     inline iterator get_iterator(size_type block_index, size_type start_row)
     {
         typename blocks_type::iterator block_pos = m_blocks.begin();
@@ -850,6 +1023,13 @@ private:
         return iterator(block_pos, m_blocks.end(), start_row, block_index);
     }
 
+    inline const_iterator get_const_iterator(size_type block_index, size_type start_row) const
+    {
+        typename blocks_type::const_iterator block_pos = m_blocks.begin();
+        std::advance(block_pos, block_index);
+        return const_iterator(block_pos, m_blocks.end(), start_row, block_index);
+    }
+
 private:
     blocks_type m_blocks;
     size_type m_cur_size;
