@@ -25,17 +25,27 @@
  *
  ************************************************************************/
 
-#ifndef __MDDS_GRID_MAP_TYPES_HPP__
-#define __MDDS_GRID_MAP_TYPES_HPP__
+#ifndef MDDS_MULTI_TYPE_VECTOR_TYPES_HPP
+#define MDDS_MULTI_TYPE_VECTOR_TYPES_HPP
 
 #include "default_deleter.hpp"
 #include "compat/unique_ptr.hpp"
 #include "global.hpp"
 
+#ifdef MDDS_MULTI_TYPE_VECTOR_USE_DEQUE
+#include <deque>
+#else
 #include <vector>
-#include <iostream>
+#endif
 #include <boost/noncopyable.hpp>
 
+#if defined(MDDS_UNIT_TEST) || defined (MDDS_MULTI_TYPE_VECTOR_DEBUG)
+#include <iostream>
+using std::cout;
+using std::cerr;
+using std::endl;
+#endif
+
 namespace mdds { namespace mtv {
 
 typedef int element_t;
@@ -51,6 +61,8 @@ const element_t element_type_uint    = 5;
 const element_t element_type_long    = 6;
 const element_t element_type_ulong   = 7;
 const element_t element_type_boolean = 8;
+const element_t element_type_char    = 9;
+const element_t element_type_uchar   = 10;
 
 const element_t element_type_user_start = 50;
 
@@ -66,17 +78,23 @@ public:
 struct base_element_block;
 element_t get_block_type(const base_element_block&);
 
+/**
+ * Non-template common base type necessary for blocks of all types to be
+ * stored in a single container.
+ */
 struct base_element_block
 {
     friend element_t get_block_type(const base_element_block&);
 protected:
     element_t type;
     base_element_block(element_t _t) : type(_t) {}
+    ~base_element_block() {}
 };
 
 template<typename _Self, element_t _TypeId, typename _Data>
 class element_block : public base_element_block
 {
+#ifdef MDDS_UNIT_TEST
     struct print_block_array
     {
         void operator() (const _Data& val) const
@@ -84,9 +102,14 @@ class element_block : public base_element_block
             std::cout << val << " ";
         }
     };
+#endif
 
 protected:
+#ifdef MDDS_MULTI_TYPE_VECTOR_USE_DEQUE
+    typedef std::deque<_Data> store_type;
+#else
     typedef std::vector<_Data> store_type;
+#endif
     store_type m_array;
 
     element_block() : base_element_block(_TypeId) {}
@@ -94,6 +117,8 @@ protected:
     element_block(size_t n, const _Data& val) : base_element_block(_TypeId), m_array(n, val) {}
 
 public:
+    static const element_t block_type = _TypeId;
+
     typedef typename store_type::iterator iterator;
     typedef typename store_type::reverse_iterator reverse_iterator;
     typedef typename store_type::const_iterator const_iterator;
@@ -110,6 +135,16 @@ public:
         return !operator==(r);
     }
 
+    static const value_type& at(const base_element_block& block, typename store_type::size_type pos)
+    {
+        return get(block).m_array.at(pos);
+    }
+
+    static value_type& at(base_element_block& block, typename store_type::size_type pos)
+    {
+        return get(block).m_array.at(pos);
+    }
+
     static iterator begin(base_element_block& block)
     {
         return get(block).m_array.begin();
@@ -204,12 +239,16 @@ public:
         static_cast<_Self&>(blk).m_array.resize(new_size);
     }
 
+#ifdef MDDS_UNIT_TEST
     static void print_block(const base_element_block& blk)
     {
         const store_type& blk2 = get(blk).m_array;
         std::for_each(blk2.begin(), blk2.end(), print_block_array());
         std::cout << std::endl;
     }
+#else
+    static void print_block(const base_element_block&) {}
+#endif
 
     static void erase_block(base_element_block& blk, size_t pos)
     {
@@ -235,12 +274,11 @@ public:
     {
         store_type& d = get(dest).m_array;
         const store_type& s = get(src).m_array;
-        typename store_type::const_iterator it = s.begin();
-        std::advance(it, begin_pos);
-        typename store_type::const_iterator it_end = it;
-        std::advance(it_end, len);
+        std::pair<const_iterator,const_iterator> its = get_iterator_pair(s, begin_pos, len);
+#ifndef MDDS_MULTI_TYPE_VECTOR_USE_DEQUE
         d.reserve(d.size() + len);
-        std::copy(it, it_end, std::back_inserter(d));
+#endif
+        std::copy(its.first, its.second, std::back_inserter(d));
     }
 
     static void assign_values_from_block(
@@ -248,12 +286,43 @@ public:
     {
         store_type& d = get(dest).m_array;
         const store_type& s = get(src).m_array;
-        assert(begin_pos + len <= s.size());
-        typename store_type::const_iterator it = s.begin();
-        std::advance(it, begin_pos);
-        typename store_type::const_iterator it_end = it;
-        std::advance(it_end, len);
-        d.assign(it, it_end);
+        std::pair<const_iterator,const_iterator> its = get_iterator_pair(s, begin_pos, len);
+        d.assign(its.first, its.second);
+    }
+
+    static void prepend_values_from_block(
+        base_element_block& dest, const base_element_block& src, size_t begin_pos, size_t len)
+    {
+        store_type& d = get(dest).m_array;
+        const store_type& s = get(src).m_array;
+        std::pair<const_iterator,const_iterator> its = get_iterator_pair(s, begin_pos, len);
+#ifndef MDDS_MULTI_TYPE_VECTOR_USE_DEQUE
+        d.reserve(d.size() + len);
+#endif
+        d.insert(d.begin(), its.first, its.second);
+    }
+
+    static void swap_values(
+        base_element_block& blk1, base_element_block& blk2, size_t pos1, size_t pos2, size_t len)
+    {
+        store_type& st1 = get(blk1).m_array;
+        store_type& st2 = get(blk2).m_array;
+        assert(pos1 + len <= st1.size());
+        assert(pos2 + len <= st2.size());
+
+        typename store_type::iterator it1 = st1.begin(), it2 = st2.begin();
+        std::advance(it1, pos1);
+        std::advance(it2, pos2);
+        for (size_t i = 0; i < len; ++i, ++it1, ++it2)
+        {
+#ifdef MDDS_MULTI_TYPE_VECTOR_USE_DEQUE
+            std::swap(*it1, *it2);
+#else
+            value_type v1 = *it1, v2 = *it2;
+            *it1 = v2;
+            *it2 = v1;
+#endif
+        }
     }
 
     template<typename _Iter>
@@ -294,6 +363,18 @@ public:
         store_type& blk = get(block).m_array;
         blk.insert(blk.begin()+pos, it_begin, it_end);
     }
+
+private:
+    static std::pair<const_iterator,const_iterator>
+    get_iterator_pair(const store_type& array, size_t begin_pos, size_t len)
+    {
+        assert(begin_pos + len <= array.size());
+        const_iterator it = array.begin();
+        std::advance(it, begin_pos);
+        const_iterator it_end = it;
+        std::advance(it_end, len);
+        return std::pair<const_iterator,const_iterator>(it, it_end);
+    }
 };
 
 template<typename _Self, element_t _TypeId, typename _Data>
@@ -385,7 +466,9 @@ struct managed_element_block : public copyable_element_block<managed_element_blo
     managed_element_block(size_t n) : base_type(n) {}
     managed_element_block(const managed_element_block& r)
     {
+#ifndef MDDS_MULTI_TYPE_VECTOR_USE_DEQUE
         m_array.reserve(r.m_array.size());
+#endif
         typename managed_element_block::store_type::const_iterator it = r.m_array.begin(), it_end = r.m_array.end();
         for (; it != it_end; ++it)
             m_array.push_back(new _Data(**it));
@@ -467,6 +550,8 @@ typedef default_element_block<mtv::element_type_uint, unsigned int>     uint_ele
 typedef default_element_block<mtv::element_type_long, long>             long_element_block;
 typedef default_element_block<mtv::element_type_ulong, unsigned long>   ulong_element_block;
 typedef default_element_block<mtv::element_type_boolean, bool>          boolean_element_block;
+typedef default_element_block<mtv::element_type_char, char>             char_element_block;
+typedef default_element_block<mtv::element_type_uchar, unsigned char>   uchar_element_block;
 
 }}
 
