@@ -30,6 +30,9 @@
 #define MDDS_MULTI_TYPE_VECTOR_DEBUG 1
 #include <mdds/multi_type_vector.hpp>
 #include <mdds/multi_type_vector_trait.hpp>
+#include <mdds/multi_type_vector_custom_func1.hpp>
+#include <mdds/multi_type_vector_custom_func2.hpp>
+#include <mdds/multi_type_vector_custom_func3.hpp>
 
 #include <cassert>
 #include <sstream>
@@ -48,6 +51,10 @@ namespace {
 /** custom cell type definition. */
 const mtv::element_t element_type_user_block  = mtv::element_type_user_start;
 const mtv::element_t element_type_muser_block = mtv::element_type_user_start+1;
+const mtv::element_t element_type_fruit_block = mtv::element_type_user_start+2;
+const mtv::element_t element_type_date_block  = mtv::element_type_user_start+3;
+
+enum my_fruit_type { unknown_fruit = 0, apple, orange, mango, peach };
 
 /** Caller manages the life cycle of these cells. */
 struct user_cell
@@ -71,8 +78,15 @@ struct muser_cell
     muser_cell(double _v) : value(_v) {}
 };
 
-typedef mdds::mtv::default_element_block<element_type_user_block, user_cell*> user_cell_block;
-typedef mdds::mtv::managed_element_block<element_type_muser_block, muser_cell> muser_cell_block;
+struct date
+{
+    int year;
+    int month;
+    int day;
+
+    date() : year(0), month(0), day(0) {}
+    date(int _year, int _month, int _day) : year(_year), month(_month), day(_day) {}
+};
 
 template<typename T>
 class cell_pool : boost::noncopyable
@@ -99,216 +113,25 @@ public:
     }
 };
 
+typedef mdds::mtv::default_element_block<element_type_user_block, user_cell*> user_cell_block;
+typedef mdds::mtv::managed_element_block<element_type_muser_block, muser_cell> muser_cell_block;
+typedef mdds::mtv::default_element_block<element_type_fruit_block, my_fruit_type> fruit_block;
+typedef mdds::mtv::default_element_block<element_type_date_block, date> date_block;
+
 MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(user_cell, element_type_user_block, NULL, user_cell_block)
 MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(muser_cell, element_type_muser_block, NULL, muser_cell_block)
+MDDS_MTV_DEFINE_ELEMENT_CALLBACKS(my_fruit_type, element_type_fruit_block, unknown_fruit, fruit_block)
+MDDS_MTV_DEFINE_ELEMENT_CALLBACKS(date, element_type_date_block, date(), date_block)
 
 }
 
-struct my_cell_block_func
-{
-    static mdds::mtv::base_element_block* create_new_block(
-        mdds::mtv::element_t type, size_t init_size)
-    {
-        switch (type)
-        {
-            case element_type_user_block:
-                return user_cell_block::create_block(init_size);
-            case element_type_muser_block:
-                return muser_cell_block::create_block(init_size);
-            default:
-                ;
-        }
-
-        return mtv::element_block_func_base::create_new_block(type, init_size);
-    }
-
-    static mdds::mtv::base_element_block* clone_block(const mdds::mtv::base_element_block& block)
-    {
-        switch (mtv::get_block_type(block))
-        {
-            case element_type_user_block:
-                return user_cell_block::clone_block(block);
-            case element_type_muser_block:
-                return muser_cell_block::clone_block(block);
-            default:
-                ;
-        }
-
-        return mtv::element_block_func_base::clone_block(block);
-    }
-
-    static void delete_block(mdds::mtv::base_element_block* p)
-    {
-        if (!p)
-            return;
-
-        switch (mtv::get_block_type(*p))
-        {
-            case element_type_user_block:
-                user_cell_block::delete_block(p);
-            break;
-            case element_type_muser_block:
-                muser_cell_block::delete_block(p);
-            break;
-            default:
-                mtv::element_block_func_base::delete_block(p);
-        }
-    }
-
-    static void resize_block(mdds::mtv::base_element_block& block, size_t new_size)
-    {
-        switch (mtv::get_block_type(block))
-        {
-            case element_type_user_block:
-                user_cell_block::resize_block(block, new_size);
-            break;
-            case element_type_muser_block:
-                muser_cell_block::resize_block(block, new_size);
-            break;
-            default:
-                mtv::element_block_func_base::resize_block(block, new_size);
-        }
-    }
-
-    static void print_block(const mdds::mtv::base_element_block& block)
-    {
-        switch (mtv::get_block_type(block))
-        {
-            case element_type_user_block:
-                user_cell_block::print_block(block);
-            break;
-            case element_type_muser_block:
-                muser_cell_block::print_block(block);
-            break;
-            default:
-                mtv::element_block_func_base::print_block(block);
-        }
-    }
-
-    static void erase(mdds::mtv::base_element_block& block, size_t pos)
-    {
-        switch (mtv::get_block_type(block))
-        {
-            case element_type_user_block:
-                user_cell_block::erase_block(block, pos);
-            break;
-            case element_type_muser_block:
-                muser_cell_block::erase_block(block, pos);
-            break;
-            default:
-                mtv::element_block_func_base::erase(block, pos);
-        }
-    }
-
-    static void erase(mdds::mtv::base_element_block& block, size_t pos, size_t size)
-    {
-        switch (mtv::get_block_type(block))
-        {
-            case element_type_user_block:
-                user_cell_block::erase_block(block, pos, size);
-            break;
-            case element_type_muser_block:
-                muser_cell_block::erase_block(block, pos, size);
-            break;
-            default:
-                mtv::element_block_func_base::erase(block, pos, size);
-        }
-    }
-
-    static void append_values_from_block(
-        mdds::mtv::base_element_block& dest, const mdds::mtv::base_element_block& src)
-    {
-        switch (mtv::get_block_type(dest))
-        {
-            case element_type_user_block:
-                user_cell_block::append_values_from_block(dest, src);
-            break;
-            case element_type_muser_block:
-                muser_cell_block::append_values_from_block(dest, src);
-            break;
-            default:
-                mtv::element_block_func_base::append_values_from_block(dest, src);
-        }
-    }
-
-    static void append_values_from_block(
-        mdds::mtv::base_element_block& dest, const mdds::mtv::base_element_block& src,
-        size_t begin_pos, size_t len)
-    {
-        switch (mtv::get_block_type(dest))
-        {
-            case element_type_user_block:
-                user_cell_block::append_values_from_block(dest, src, begin_pos, len);
-            break;
-            case element_type_muser_block:
-                muser_cell_block::append_values_from_block(dest, src, begin_pos, len);
-            break;
-            default:
-                mtv::element_block_func_base::append_values_from_block(dest, src, begin_pos, len);
-        }
-    }
-
-    static void assign_values_from_block(
-        mdds::mtv::base_element_block& dest, const mdds::mtv::base_element_block& src,
-        size_t begin_pos, size_t len)
-    {
-        switch (mtv::get_block_type(dest))
-        {
-            case element_type_user_block:
-                user_cell_block::assign_values_from_block(dest, src, begin_pos, len);
-            break;
-            case element_type_muser_block:
-                muser_cell_block::assign_values_from_block(dest, src, begin_pos, len);
-            break;
-            default:
-                mtv::element_block_func_base::assign_values_from_block(dest, src, begin_pos, len);
-        }
-    }
-
-    static bool equal_block(
-        const mdds::mtv::base_element_block& left, const mdds::mtv::base_element_block& right)
-    {
-        if (mtv::get_block_type(left) == element_type_user_block)
-        {
-            if (mtv::get_block_type(right) != element_type_user_block)
-                return false;
-
-            return user_cell_block::get(left) == user_cell_block::get(right);
-        }
-        else if (mtv::get_block_type(right) == element_type_user_block)
-            return false;
-
-        if (mtv::get_block_type(left) == element_type_muser_block)
-        {
-            if (mtv::get_block_type(right) != element_type_muser_block)
-                return false;
-
-            return muser_cell_block::get(left) == muser_cell_block::get(right);
-        }
-        else if (mtv::get_block_type(right) == element_type_muser_block)
-            return false;
-
-        return mtv::element_block_func_base::equal_block(left, right);
-    }
-
-    static void overwrite_values(mdds::mtv::base_element_block& block, size_t pos, size_t len)
-    {
-        switch (mtv::get_block_type(block))
-        {
-            case element_type_user_block:
-                // Do nothing.  The client code manages the life cycle of these cells.
-            break;
-            case element_type_muser_block:
-                muser_cell_block::overwrite_values(block, pos, len);
-            break;
-            default:
-                mtv::element_block_func_base::overwrite_values(block, pos, len);
-        }
-    }
-};
-
 namespace {
 
+typedef multi_type_vector<mtv::custom_block_func2<user_cell_block, muser_cell_block> > mtv_type;
+typedef multi_type_vector<mtv::custom_block_func1<fruit_block> > mtv_fruit_type;
+typedef multi_type_vector<
+    mtv::custom_block_func3<muser_cell_block, fruit_block, date_block> > mtv3_type;
+
 template<typename _ColT, typename _ValT>
 bool test_cell_insertion(_ColT& col_db, size_t row, _ValT val)
 {
@@ -318,8 +141,6 @@ bool test_cell_insertion(_ColT& col_db, size_t row, _ValT val)
     return val == test;
 }
 
-typedef mdds::multi_type_vector<my_cell_block_func> mtv_type;
-
 void mtv_test_types()
 {
     stack_printer __stack_printer__("::mtv_test_types");
@@ -342,6 +163,19 @@ void mtv_test_types()
     user_cell* p = NULL;
     ct = mtv_type::get_element_type(p);
     assert(ct == element_type_user_block && ct >= mtv::element_type_user_start);
+    ct = mtv_type::get_element_type(static_cast<muser_cell*>(NULL));
+    assert(ct == element_type_muser_block && ct >= mtv::element_type_user_start);
+    ct = mtv_fruit_type::get_element_type(unknown_fruit);
+    assert(ct == element_type_fruit_block && ct >= mtv::element_type_user_start);
+}
+
+void mtv_test_block_identifier()
+{
+    stack_printer __stack_printer__("::mtv_test_block_identifier");
+    assert(user_cell_block::block_type == element_type_user_block);
+    assert(muser_cell_block::block_type == element_type_muser_block);
+    assert(fruit_block::block_type == element_type_fruit_block);
+    assert(date_block::block_type == element_type_date_block);
 }
 
 void mtv_test_basic()
@@ -1048,27 +882,574 @@ void mtv_test_managed_block()
         db.set_empty(1, 1);
         assert(db.block_size() == 1);
     }
+
+    {
+        // Release an element.
+        mtv_type db(1);
+        muser_cell* p1 = new muser_cell(4.5);
+        db.set(0, p1);
+        muser_cell* p2 = db.release<muser_cell*>(0);
+        assert(p1 == p2);
+        assert(p2->value == 4.5);
+        assert(db.is_empty(0));
+        delete p2;
+
+        db = mtv_type(2);
+        db.set(0, new muser_cell(23.3));
+        assert(db.block_size() == 2);
+        p2 = db.release<muser_cell*>(0);
+        assert(db.is_empty(0));
+        assert(db.is_empty(1));
+        assert(db.block_size() == 1);
+        delete p2;
+
+        db = mtv_type(2);
+        db.set(0, new muser_cell(1.2));
+        db.set(1, new muser_cell(1.3));
+
+        p2 = db.release<muser_cell*>(0);
+        assert(db.is_empty(0));
+        assert(!db.is_empty(1));
+        assert(p2->value == 1.2);
+        delete p2;
+
+        db.set(0, new muser_cell(1.4));
+        p2 = db.release<muser_cell*>(1);
+        assert(!db.is_empty(0));
+        assert(db.is_empty(1));
+        assert(p2->value == 1.3);
+        delete p2;
+
+        db = mtv_type(3);
+        db.set(0, new muser_cell(2.1));
+        db.set(1, new muser_cell(2.2));
+        db.set(2, new muser_cell(2.3));
+
+        p2 = db.release<muser_cell*>(1);
+        assert(p2->value == 2.2);
+        assert(!db.is_empty(0));
+        assert(db.is_empty(1));
+        assert(!db.is_empty(2));
+
+        delete p2;
+
+        db = mtv_type(3);
+        db.set(0, new muser_cell(2.1));
+        db.set(1, new muser_cell(2.2));
+        db.set(2, new muser_cell(2.3));
+        db.set_empty(0, 2); // Make sure this doesn't release anything.
+
+        // Release with position hint.
+        db = mtv_type(4);
+        db.set(0, new muser_cell(4.5));
+        db.set(1, new muser_cell(4.6));
+        db.set(3, new muser_cell(5.1));
+
+        mtv_type::iterator pos = db.release(0, p1);
+        assert(pos == db.begin());
+        pos = db.release(pos, 3, p2);
+        ++pos;
+        assert(pos == db.end());
+        assert(p1->value == 4.5);
+        assert(p2->value == 5.1);
+        assert(db.block_size() == 3);
+        assert(db.is_empty(0));
+        assert(db.get<muser_cell*>(1)->value == 4.6);
+        assert(db.is_empty(2));
+        assert(db.is_empty(3));
+        delete p1;
+        delete p2;
+    }
+}
+
+void mtv_test_custom_block_func1()
+{
+    stack_printer __stack_printer__("::mtv_test_custom_block_func1");
+    mtv_fruit_type db(10);
+    db.set(0, apple);
+    db.set(1, orange);
+    db.set(2, mango);
+    db.set(3, peach);
+    assert(db.block_size() == 2);
+    assert(db.get_type(0) == element_type_fruit_block);
+    assert(db.get<my_fruit_type>(0) == apple);
+    assert(db.get<my_fruit_type>(1) == orange);
+    assert(db.get<my_fruit_type>(2) == mango);
+    assert(db.get<my_fruit_type>(3) == peach);
+    db.set<int>(1, 234);
+    assert(db.block_size() == 4);
+    db.set(1, apple);
+    assert(db.block_size() == 2);
+}
+
+void mtv_test_transfer()
+{
+    stack_printer __stack_printer__("::mtv_test_transfer");
+    mtv_type db1(3), db2(4); // db2 is larger than db1.
+    db1.set(0, new muser_cell(1.1));
+    db1.set(1, new muser_cell(1.2));
+    db1.set(2, new muser_cell(1.3));
+    assert(db1.block_size() == 1);
+
+    try
+    {
+        db1.transfer(0, 1, db1, 0);
+        assert(!"Exception should have been thrown");
+    }
+    catch (const invalid_arg_error&)
+    {
+        // Good.
+    }
+
+    // Do the transfer.
+    db1.transfer(0, 2, db2, 0);
+
+    // Now db1 should be totally empty.
+    assert(db1.block_size() == 1);
+    mtv_type::iterator check = db1.begin();
+    assert(check != db1.end());
+    assert(check->type == mtv::element_type_empty);
+    assert(check->size == 3);
+
+    assert(db2.block_size() == 2);
+    assert(db2.get<muser_cell*>(0)->value == 1.1);
+    assert(db2.get<muser_cell*>(1)->value == 1.2);
+    assert(db2.get<muser_cell*>(2)->value == 1.3);
+    assert(db2.is_empty(3));
+
+    // Transfer back to db1. This should make db2 to be totally empty again.
+    db2.transfer(0, 2, db1, 0);
+    assert(db2.block_size() == 1);
+    check = db2.begin();
+    assert(check != db2.end());
+    assert(check->size == 4);
+    assert(check->type == mtv::element_type_empty);
+
+    assert(db1.block_size() == 1);
+    assert(db1.get<muser_cell*>(0)->value == 1.1);
+    assert(db1.get<muser_cell*>(1)->value == 1.2);
+    assert(db1.get<muser_cell*>(2)->value == 1.3);
+
+    // Now, transfer only the top 2 elements.
+    db1.transfer(0, 1, db2, 0);
+    assert(db1.is_empty(0));
+    assert(db1.is_empty(1));
+    assert(db1.get<muser_cell*>(2)->value == 1.3);
+
+    assert(db2.get<muser_cell*>(0)->value == 1.1);
+    assert(db2.get<muser_cell*>(1)->value == 1.2);
+    assert(db2.is_empty(2));
+    assert(db2.is_empty(3));
+
+    // .. and back.
+    db2.transfer(0, 1, db1, 0);
+    assert(db1.block_size() == 1);
+    assert(db1.get<muser_cell*>(0)->value == 1.1);
+    assert(db1.get<muser_cell*>(1)->value == 1.2);
+    assert(db1.get<muser_cell*>(2)->value == 1.3);
+
+    assert(db2.block_size() == 1);
+    check = db2.begin();
+    assert(check != db2.end());
+    assert(check->size == 4);
+    assert(check->type == mtv::element_type_empty);
+
+    db1 = mtv_type(4);
+    db2 = mtv_type(4);
+    db2.set(1, new muser_cell(2.1));
+    db2.set(2, new muser_cell(2.2));
+    assert(db2.block_size() == 3);
+    db1.transfer(0, 1, db2, 1); // This causes db2's 3 blocks to merge into one.
+    assert(db2.block_size() == 1);
+    check = db2.begin();
+    assert(check != db2.end());
+    assert(check->size == 4);
+    assert(check->type == mtv::element_type_empty);
+    assert(db1.block_size() == 1);
+    check = db1.begin();
+    assert(check != db1.end());
+    assert(check->size == 4);
+    assert(check->type == mtv::element_type_empty);
+
+    db2.set(0, new muser_cell(3.1));
+    assert(db2.block_size() == 2);
+
+    db1.set(1, new muser_cell(3.2));
+    db1.set(2, new muser_cell(3.3));
+    db1.set(3, new muser_cell(3.4));
+    assert(db1.block_size() == 2);
+
+    db1.transfer(1, 2, db2, 1);
+    assert(db1.block_size() == 2);
+    assert(db1.is_empty(0));
+    assert(db1.is_empty(1));
+    assert(db1.is_empty(2));
+    assert(db1.get<muser_cell*>(3)->value == 3.4);
+    assert(db2.block_size() == 2);
+    assert(db2.get<muser_cell*>(0)->value == 3.1);
+    assert(db2.get<muser_cell*>(1)->value == 3.2);
+    assert(db2.get<muser_cell*>(2)->value == 3.3);
+    assert(db2.is_empty(3));
+
+    db1 = mtv_type(3);
+    db2 = mtv_type(3);
+    db1.set(1, new muser_cell(4.2));
+    db2.set(0, new muser_cell(4.1));
+    db2.set(2, new muser_cell(4.3));
+
+    db1.transfer(1, 1, db2, 1);
+    assert(db1.block_size() == 1);
+    check = db1.begin();
+    assert(check != db1.end());
+    assert(check->size == 3);
+    assert(check->type == mtv::element_type_empty);
+    assert(db2.block_size() == 1);
+    assert(db2.get<muser_cell*>(0)->value == 4.1);
+    assert(db2.get<muser_cell*>(1)->value == 4.2);
+    assert(db2.get<muser_cell*>(2)->value == 4.3);
+
+    // Transfer to middle of block.
+    db1 = mtv_type(3);
+    db2 = mtv_type(3);
+    db1.set(0, new muser_cell(5.2));
+    assert(db1.block_size() == 2);
+    db1.transfer(0, 0, db2, 1);
+    assert(db1.block_size() == 1);
+    check = db1.begin();
+    assert(check != db1.end());
+    assert(check->size == 3);
+    assert(check->type == mtv::element_type_empty);
+    assert(db2.block_size() == 3);
+    assert(db2.is_empty(0));
+    assert(db2.get<muser_cell*>(1)->value == 5.2);
+    assert(db2.is_empty(2));
+
+    db1 = mtv_type(2);
+    db2 = mtv_type(3);
+    db1.set(0, new muser_cell(11.1));
+    db1.set(1, new muser_cell(11.2));
+    db1.transfer(1, 1, db2, 1);
+    assert(db1.block_size() == 2);
+    assert(db1.get<muser_cell*>(0)->value == 11.1);
+    assert(db1.is_empty(1));
+    assert(db2.block_size() == 3);
+    assert(db2.is_empty(0));
+    assert(db2.get<muser_cell*>(1)->value == 11.2);
+    assert(db2.is_empty(2));
+
+    // Transfer to bottom of block.
+    db1 = mtv_type(4);
+    db2 = mtv_type(5);
+    db1.set(0, new muser_cell(6.1));
+    db1.set(1, new muser_cell(6.2));
+    db1.transfer(0, 1, db2, 3);
+    assert(db1.block_size() == 1);
+    check = db1.begin();
+    assert(check != db1.end());
+    assert(check->size == 4);
+    assert(check->type == mtv::element_type_empty);
+    assert(db2.block_size() == 2);
+    assert(db2.is_empty(0));
+    assert(db2.is_empty(1));
+    assert(db2.is_empty(2));
+    assert(db2.get<muser_cell*>(3)->value == 6.1);
+    assert(db2.get<muser_cell*>(4)->value == 6.2);
+
+    // Transfer multiple blocks.  Very simple use case.
+    db1 = mtv_type(4);
+    db2 = mtv_type(3);
+    db1.set(1, new muser_cell(10.1));
+    db1.set(3, new muser_cell(10.2));
+    db1.transfer(1, 3, db2, 0);
+
+    // db1 should be completely empty.
+    assert(db1.block_size() == 1);
+    check = db1.begin();
+    assert(check != db1.end());
+    assert(check->size == 4);
+    assert(check->type == mtv::element_type_empty);
+
+    assert(db2.block_size() == 3);
+    assert(db2.get<muser_cell*>(0)->value == 10.1);
+    assert(db2.is_empty(1));
+    assert(db2.get<muser_cell*>(2)->value == 10.2);
+
+    // Multiple-block transfer that involves merging.
+    db1 = mtv_type(5);
+    db2 = mtv_type(5);
+    db1.set(0, new muser_cell(0.1));
+    db1.set(1, new muser_cell(0.2));
+    db1.set(3, new muser_cell(0.3));
+    db1.set(4, new muser_cell(0.4));
+
+    db2.set(0, new muser_cell(1.1));
+    db2.set(4, new muser_cell(1.2));
+
+    mtv_type::iterator it = db1.transfer(1, 3, db2, 1);
+    assert(db1.block_size() == 3);
+    assert(db1.get<muser_cell*>(0)->value == 0.1);
+    assert(db1.is_empty(1));
+    assert(db1.is_empty(2));
+    assert(db1.is_empty(3));
+    assert(db1.get<muser_cell*>(4)->value == 0.4);
+
+    assert(db2.block_size() == 3);
+    assert(db2.get<muser_cell*>(0)->value == 1.1);
+    assert(db2.get<muser_cell*>(1)->value == 0.2);
+    assert(db2.is_empty(2));
+    assert(db2.get<muser_cell*>(3)->value == 0.3);
+    assert(db2.get<muser_cell*>(4)->value == 1.2);
+
+    assert(it != db1.end());
+    assert(it->size == 3);
+    assert(it->type == mtv::element_type_empty);
+    it = db1.transfer(it, 4, 4, db2, 2); // Transfer single element at 4.
+    assert(db1.block_size() == 2);
+    assert(db1.get<muser_cell*>(0)->value == 0.1);
+    assert(db1.is_empty(1));
+    assert(db1.is_empty(2));
+    assert(db1.is_empty(3));
+    assert(db1.is_empty(4));
+
+    assert(db2.block_size() == 1);
+    assert(db2.get<muser_cell*>(0)->value == 1.1);
+    assert(db2.get<muser_cell*>(1)->value == 0.2);
+    assert(db2.get<muser_cell*>(2)->value == 0.4);
+    assert(db2.get<muser_cell*>(3)->value == 0.3);
+    assert(db2.get<muser_cell*>(4)->value == 1.2);
+
+    assert(it != db1.end());
+    assert(it->size == 4);
+    assert(it->type == mtv::element_type_empty);
+    ++it;
+    assert(it == db1.end());
+
+    // Multi-block transfer to the top part of destination block.
+    db1 = mtv_type(5);
+    db2 = mtv_type(5);
+    db1.set(0, new muser_cell(-1.1));
+    db1.set(1, new muser_cell(-2.1));
+    db1.set(2, new muser_cell(-3.1));
+    db1.set(3, string("foo"));
+    db1.set(4, new muser_cell(-5.1));
+    db2.set(1, true);
+    db2.set(2, false);
+    db2.set(3, true);
+    it = db1.transfer(2, 3, db2, 2);
+    assert(it != db1.end());
+    assert(it->size == 2);
+    assert(it->type == mtv::element_type_empty);
+    std::advance(it, 2);
+    assert(it == db1.end());
+    assert(db1.block_size() == 3);
+    assert(db1.get<muser_cell*>(0)->value == -1.1);
+    assert(db1.get<muser_cell*>(1)->value == -2.1);
+    assert(db1.is_empty(2));
+    assert(db1.is_empty(3));
+    assert(db1.get<muser_cell*>(4)->value == -5.1);
+
+    assert(db2.block_size() == 5);
+    assert(db2.is_empty(0));
+    assert(db2.get<bool>(1) == true);
+    assert(db2.get<muser_cell*>(2)->value == -3.1);
+    assert(db2.get<string>(3) == "foo");
+    assert(db2.is_empty(4));
+
+    // Multi-block transfer to the bottom part of destination block.
+    db1 = mtv_type(10);
+    db2 = mtv_type(10);
+    db1.set(0, new muser_cell(2.1));
+    db1.set(1, new muser_cell(2.2));
+    db1.set(2, char('a'));
+    db1.set(3, char('b'));
+    db2.set(0, true);
+    db2.set(1, false);
+
+    it = db1.transfer(0, 2, db2, 7);
+    assert(it != db1.end());
+    assert(it->size == 3);
+    assert(it->type == mtv::element_type_empty);
+    ++it;
+    assert(it != db1.end());
+    assert(it->size == 1);
+    assert(it->type == mtv::element_type_char);
+    ++it;
+    assert(it != db1.end());
+    assert(it->size == 6);
+    assert(it->type == mtv::element_type_empty);
+    ++it;
+    assert(it == db1.end());
+    assert(db1.block_size() == 3);
+    assert(db1.get<char>(3) == 'b');
+
+    assert(db2.block_size() == 4);
+    assert(db2.get<bool>(0) == true);
+    assert(db2.get<bool>(1) == false);
+    assert(db2.is_empty(2));
+    assert(db2.is_empty(3));
+    assert(db2.is_empty(4));
+    assert(db2.is_empty(5));
+    assert(db2.is_empty(6));
+    assert(db2.get<muser_cell*>(7)->value == 2.1);
+    assert(db2.get<muser_cell*>(8)->value == 2.2);
+    assert(db2.get<char>(9) == 'a');
+
+    // Multi-block transfer to the middle part of destination block.
+    db1 = mtv_type(10);
+    db2 = mtv_type(10);
+    db2.set(0, true);
+    db2.set(9, true);
+    db1.set(3, new muser_cell(2.4));
+    db1.set(4, new muser_cell(2.5));
+    db1.set(5, string("abc"));
+    db1.set(6, new muser_cell(2.6));
+    db1.set(7, new muser_cell(2.7));
+    db1.set(8, true);
+    it = db1.transfer(3, 6, db2, 2);
+    assert(it != db1.end());
+    assert(it->size == 7);
+    assert(it->type == mtv::element_type_empty);
+    ++it;
+    assert(it != db1.end());
+    assert(it->size == 1);
+    assert(it->type == element_type_muser_block);
+    ++it;
+    assert(it->size == 1);
+    assert(it->type == mtv::element_type_boolean);
+    ++it;
+    assert(it->size == 1);
+    assert(it->type == mtv::element_type_empty);
+    ++it;
+    assert(it == db1.end());
+
+    assert(db1.block_size() == 4);
+    assert(db1.is_empty(6));
+    assert(db1.get<muser_cell*>(7)->value == 2.7);
+    assert(db1.get<bool>(8) == true);
+    assert(db1.is_empty(9));
+
+    assert(db2.block_size() == 7);
+    assert(db2.get<bool>(0) == true);
+    assert(db2.is_empty(1));
+    assert(db2.get<muser_cell*>(2)->value == 2.4);
+    assert(db2.get<muser_cell*>(3)->value == 2.5);
+    assert(db2.get<string>(4) == "abc");
+    assert(db2.get<muser_cell*>(5)->value == 2.6);
+    assert(db2.is_empty(6));
+    assert(db2.is_empty(7));
+    assert(db2.is_empty(8));
+    assert(db2.get<bool>(9) == true);
+
+    db1 = mtv_type(10);
+    db2 = mtv_type(10);
+    db1.set(3, true);
+    db2.set(3, string("test"));
+    db1.transfer(0, 6, db2, 0);
+    assert(db1.block_size() == 1);
+    check = db1.begin();
+    assert(check != db1.end());
+    assert(check->size == 10);
+    assert(check->type == mtv::element_type_empty);
+    ++check;
+    assert(check == db1.end());
+    assert(db2.block_size() == 3);
+    assert(db2.is_empty(0));
+    assert(db2.is_empty(1));
+    assert(db2.is_empty(2));
+    assert(db2.get<bool>(3) == true);
+    assert(db2.is_empty(4));
+    assert(db2.is_empty(5));
+    assert(db2.is_empty(6));
+    assert(db2.is_empty(7));
+    assert(db2.is_empty(8));
+    assert(db2.is_empty(9));
+
+    // Make sure that transfer will overwrite cells in managed blocks.
+    db1 = mtv_type(3);
+    db2 = mtv_type(3);
+    db1.set(0, new muser_cell(1.1));
+    db1.set(1, new muser_cell(1.2));
+    db1.set(2, new muser_cell(1.3));
+
+    db2.set(1, new muser_cell(2.1)); // This element will be overwritten.
+    db1.transfer(0, 2, db2, 0);
+    assert(db1.is_empty(0));
+    assert(db1.is_empty(1));
+    assert(db1.is_empty(2));
+    assert(db2.get<muser_cell*>(0)->value == 1.1);
+    assert(db2.get<muser_cell*>(1)->value == 1.2);
+    assert(db2.get<muser_cell*>(2)->value == 1.3);
+}
+
+void mtv_test_swap()
+{
+    stack_printer __stack_printer__("::mtv_test_swap");
+    mtv3_type db1(5), db2(2);
+    db1.set(0, new muser_cell(1.1));
+    db1.set(1, new muser_cell(1.2));
+    db1.set(2, new muser_cell(1.3));
+    db1.set(3, new muser_cell(1.4));
+    db1.set(4, new muser_cell(1.5));
+
+    db2.set(0, string("A"));
+    db2.set(1, string("B"));
+
+    db1.swap(2, 3, db2, 0);
+}
+
+void mtv_test_custom_block_func3()
+{
+    stack_printer __stack_printer__("::mtv_test_custom_block_func3");
+    mtv3_type db(10);
+
+    // Insert custom elements.
+    db.set(0, new muser_cell(12.3));
+    db.set(1, apple);
+    db.set(2, date(1989,12,13));
+    db.set(3, date(2011,8,7));
+    assert(db.get_type(0) == element_type_muser_block);
+    assert(db.get_type(1) == element_type_fruit_block);
+    assert(db.get_type(2) == element_type_date_block);
+    assert(db.get_type(3) == element_type_date_block);
+    assert(db.get<muser_cell*>(0)->value == 12.3);
+    assert(db.get<my_fruit_type>(1) == apple);
+    assert(db.get<date>(2).year == 1989);
+    assert(db.get<date>(2).month == 12);
+    assert(db.get<date>(2).day == 13);
+    assert(db.get<date>(3).year == 2011);
+    assert(db.get<date>(3).month == 8);
+    assert(db.get<date>(3).day == 7);
+    assert(db.block_size() == 4);
+
+    // We should still support the primitive types.
+    db.set(8, 34.56);
+    assert(db.get<double>(8) == 34.56);
+}
+
+void mtv_test_release()
+{
+    stack_printer __stack_printer__("::mtv_test_release");
+    mtv_type db(3);
+    muser_cell c1(1.2), c2(1.3);
+    db.set(0, &c1);
+    db.set(2, &c2);
+    db.release(); // Prevent invalid free when db goes out of scope.
 }
 
 }
 
 int main (int argc, char **argv)
 {
-    cmd_options opt;
-    if (!parse_cmd_options(argc, argv, opt))
-        return EXIT_FAILURE;
-
-    if (opt.test_func)
-    {
-        mtv_test_types();
-        mtv_test_basic();
-        mtv_test_equality();
-        mtv_test_managed_block();
-    }
-
-    if (opt.test_perf)
-    {
-    }
+    mtv_test_types();
+    mtv_test_block_identifier();
+    mtv_test_basic();
+    mtv_test_equality();
+    mtv_test_managed_block();
+    mtv_test_custom_block_func1();
+    mtv_test_transfer();
+    mtv_test_swap();
+    mtv_test_custom_block_func3();
+    mtv_test_release();
 
     cout << "Test finished successfully!" << endl;
     return EXIT_SUCCESS;
